from medialib import *
from math import ceil
from statistics import median
# from matplotlib import pyplot as plt
initialize()
# load image and draw pixels
pixel_size = 2      # size of each pixel when drawing on the window
pixels_gray = []    # to store the pixel values
pixel_range = 2     # range of pxiel value for each bar of the histogram
red = [0 for i in range(ceil(256/pixel_range))]     # to store the count of pixels in each range of pixel values (red)
green = [0 for i in range(ceil(256/pixel_range))]   # to store the count of pixels in each range of pixel values (green)
blue = [0 for i in range(ceil(256/pixel_range))]    # to store the count of pixels in each range of pixel values (blue)
with open("cat.ppm") as f: # open the image file
    f.readline() # 1st line (P3)
    ncol, nrow = f.readline().split() # 2nd line (100 75)
    max_pixel = int(f.readline()) # 3rd line (255)
    nrow = int(nrow)
    ncol = int(ncol)
    for i in range(nrow): # loop through the rows of pixels
        row = []
        values = [int(p) for p in f.readline().split()]
        for j in range(ncol): # loop through the columns of a row
            r = values[j * 3] # get the r, g, b values of a pixel
            g = values[j * 3 + 1]
            b = values[j * 3 + 2]
            rect(j * pixel_size, i * pixel_size, pixel_size, pixel_size, r, g, b) # draw the pixel
            red[r//pixel_range] += 1    # accumulate the pixel count of the corresponded pixel range
            green[g//pixel_range] += 1
            blue[b//pixel_range] += 1
            gray = r * 0.299 + g * 0.587 + b * 0.114 # convert the rgb value to grayscale
            row.append(round(gray)) # store the grayscale pixel value to a list
        pixels_gray.append(row)
    refresh()

# function to draw a histogram
# variables:
#   x, y: origin(top-left) coordinates of the graph
#   w, h: width and height (in pixels) of the graph
#   data: a list of the counts of each range
#   max_value: the value corresponding to the height of the graph (max height of the bars)
#   r, g, b: color of the bars
def plot_histogram(x, y, w, h, data, max_value, r, g, b):
    bar_width = w / len(data)  # calculate the width of each bar
    for i in range(len(data)):
        val = data[i]
        bar_height = val / max_value * h  # calculate the height of the bar
        rect(x + bar_width * i, y + h - bar_height, bar_width, bar_height, r, g, b)  # draw the bar

wait_mouse_leftclick()
# get the greatest count of pixels of all the pixel value ranges among all the colors
rgb_max = max([max(red), max(green), max(blue)])
# draw histograms of r, g and b
plot_histogram(400, 0, 400, 300, red, rgb_max, 255, 0, 0)
plot_histogram(0, 300, 400, 300, green, rgb_max, 0, 255, 0)
plot_histogram(400, 300, 400, 300, blue, rgb_max, 0, 0, 255)

# write the grayscale image to a .pgm file
with open("cat.pgm", "w") as f:
    f.write("P2\n") # write the 1st line (P2)
    f.write(str(ncol) + " " + str(nrow) + "\n") # write the 2nd line (100 75)
    f.write("255\n") # write the 3rd line (255)
    for row in pixels_gray: # write the pixels
        for p in row:
            f.write(str(p) + " ")
        f.write("\n")

wait_mouse_leftclick()
clear()
# initialize the lists for the histograms
red = [0 for i in range(ceil(256/pixel_range))]
green = [0 for i in range(ceil(256/pixel_range))]
blue = [0 for i in range(ceil(256/pixel_range))]

# read the file of the grayscale image
with open("cat.pgm") as f:
    f.readline() # the 1st line
    ncol, nrow = f.readline().split() # the 2nd line
    max_pixel = int(f.readline()) # the 3rd line
    nrow = int(nrow)
    ncol = int(ncol)
    for i in range(nrow):
        values = [int(p) for p in f.readline().split()] # get the pixel values of the line
        for j in range(ncol):
            r = g = b = values[j]
            rect_norefresh(j * pixel_size, i * pixel_size, pixel_size, pixel_size, r, g, b) # draw the pixel
            red[r//pixel_range] += 1 # accumulate the pixel count of the corresponded pixel range
            green[g//pixel_range] += 1
            blue[b//pixel_range] += 1
    refresh()

wait_mouse_leftclick()
# draw histograms of the grayscale image
rgb_max = max([max(red), max(green), max(blue)])
plot_histogram(400, 0, 400, 300, red, rgb_max, 255, 0, 0)
plot_histogram(0, 300, 400, 300, green, rgb_max, 0, 255, 0)
plot_histogram(400, 300, 400, 300, blue, rgb_max, 0, 0, 255)
wait_mouse_leftclick()

# apply median filter to the grayscale image
filter_mask = [[x, y] for x in range(-1, 2) for y in range(-1, 2)]
pixels_filtered = [[255 for j in range(ncol)] for i in range(nrow)]
for i in range(1, nrow - 1):
    for j in range(1, ncol - 1):
        filter_window = [pixels_gray[i + mask[0]][j + mask[1]] for mask in filter_mask]
        pixels_filtered[i][j] = median(filter_window)

# write the filtered image to a .pgm file
with open("cat_filtered.pgm", "w") as f:
    f.write("P2\n") # write the 1st line (P2)
    f.write(str(ncol) + " " + str(nrow) + "\n") # write the 2nd line (100 75)
    f.write("255\n") # write the 3rd line (255)
    for row in pixels_filtered: # write the pixels
        for p in row:
            f.write(str(p) + " ")
        f.write("\n")

wait_mouse_leftclick()
clear()

# read the file of the grayscale image
with open("cat_filtered.pgm") as f:
    f.readline() # the 1st line
    ncol, nrow = f.readline().split() # the 2nd line
    max_pixel = int(f.readline()) # the 3rd line
    nrow = int(nrow)
    ncol = int(ncol)
    for i in range(nrow):
        values = [int(p) for p in f.readline().split()] # get the pixel values of the line
        for j in range(ncol):
            r = g = b = values[j]
            rect_norefresh(j * pixel_size, i * pixel_size, pixel_size, pixel_size, r, g, b) # draw the pixel
            red[r//pixel_range] += 1 # accumulate the pixel count of the corresponded pixel range
            green[g//pixel_range] += 1
            blue[b//pixel_range] += 1
    refresh()
    
wait_mouse_leftclick()
# draw histograms of the grayscale image
rgb_max = max([max(red), max(green), max(blue)])
plot_histogram(400, 0, 400, 300, red, rgb_max, 255, 0, 0)
plot_histogram(0, 300, 400, 300, green, rgb_max, 0, 255, 0)
plot_histogram(400, 300, 400, 300, blue, rgb_max, 0, 0, 255)
wait_mouse_leftclick()

all_done()
