# Plot the given data to figures with matplotlib
# Use medialib to load and show the saved image files of the figures
# Created by Min Lu, 2021-06-06, updated 2022-04-08
from matplotlib import pyplot as plt
from medialib import *

initialize() # always the first instruction of the program

def plot_bars_to_file(x, y, color, title, x_label, y_label, filename, y_limit):
    plt.bar(x, y, color=color)
    plt.xlabel(x_label)
    plt.ylabel(y_label)
    plt.ylim(0, y_limit)  # set the upper limit of Y-axis
    #plt.axis([0, len(x) + 1, 0, y_limit])  # another way to set the axes
    plt.title(title)
    #plt.show()
    plt.savefig(filename)
    plt.clf()

day = []  # days: 1, 2, 3, ...
fukuoka = []  # Number of cases increased each day in Fukuoka
tokyo = []  # Number of cases increased each day in Tokyo
osaka = []  # Number of cases increased each day in Osaka
hokkaido = []  # Number of cases increased each day in Hokkaido

# 0. load data from file
lines = open("data.csv", "r", encoding="utf-8").readlines()
for i in range(1, len(lines)):
    _nums = lines[i].split(",")
    day.append(int(_nums[0]))
    fukuoka.append(int(_nums[1]))
    tokyo.append(int(_nums[2]))
    osaka.append(int(_nums[3]))
    hokkaido.append(int(_nums[4]))

# 1. use matplotlib to plot the figures and save to files

# find the maximum value from all the cities' data
# and the upper limit of Y-axis is set to 1.1 times of the max value
y_limit = max([max(fukuoka), max(tokyo), max(osaka), max(hokkaido)]) * 1.1

plot_bars_to_file(day, fukuoka, "r", "Daily Cases in Fukuoka", "Day", "Case Count", "fukuoka.png", y_limit)
plot_bars_to_file(day, tokyo, "g", "Daily Cases in Tokyo", "Day", "Case Count", "tokyo.png", y_limit)
plot_bars_to_file(day, osaka, "b", "Daily Cases in Osaka", "Day", "Case Count", "osaka.png", y_limit)
plot_bars_to_file(day, hokkaido, "m", "Daily Cases in Hokkaido", "Day", "Case Count", "hokkaido.png", y_limit)

"""
to plot the figures.
Finally, remember to remove the above example code for a figure of Fukuoka.
"""

# 2. Use "draw" function of medialib to load and draw the figures on the window
# Example:
draw("fukuoka.png", 0, 0, 400, 300)
# Arguments: filename, x, y, width, height
# Consider the coordinates and sizes of other figures.
draw("tokyo.png", 400, 0, 400, 300)
draw("osaka.png", 0, 300, 400, 300)
draw("hokkaido.png", 400, 300, 400, 300)

wait_mouse_leftclick()

print("done")

all_done() # always the last instruction of the program
