# A program drawing horizontal bars to visualize a given list of data and their average value
# Created by Min Lu, 2021-06-25, updated 2022-04-08
# make sure medialib.py is at the same folder as this file
## magic code #########################
from medialib import *
initialize() # always the first instruction of the program
#######################################

# Draw a horizontal bar, which is a rectangle from top-left coordinates (x, y)
# with height (bar_breadth) and width (bar_length) and color (r, g, b, 0~255).
# Text of the value (number) will be drawn on the right of the bar
def draw_horizontal_bar(value, x, y, bar_breadth, bar_length, r, g, b):
    # Draw the horizontal bar (rectangle)
    rect(x, y, bar_length, bar_breadth, r, g, b)

    # Draw the text
    value_str = "{:.3f}".format(value)  # convert the value to str (3 decimal points)
    text_x = x + bar_length + 10  # calculate the top-left coordinates of the text
    text_y = y
    text(value_str, text_x, text_y, bar_breadth, r, g, b)  # draw text

# data to visualize (estimated population of Fukuoka city in
# 1985, 1990, 1995, 2000, 2005, 2010, 2015, and 2020, in M)
data = [1.160, 1.237, 1.285, 1.341, 1.401, 1.464, 1.539, 1.601]

# definitions for the bars
origin_x = 100  # the top-left coordinates of the first bar, all the bars have the same x,
origin_y = 50  # but the y coordinates should be calculated
bar_interval = 30  # distance between 2 neighboring bars ()
bar_max_length = 600  # a bar of max value 2.000 will have a max length 600
bar_max_value = 2.000  # you need to calculate the length of each bar as value / max_value * max_lengh
bar_breadth = 30  # breadth of each bar
bar_color = [255, 255, 0]  # color of the data bar and text (yellow), you can change as you like
bar_color_above = [255, 0, 0] # color of the data bar and text (red) when above average, you can change as you like
bar_color_average = [135, 206, 235]  # color of the average value bar and text (skyblue), you can change as you like

# calculate average
average = sum(data) / len(data)

# draw the bars with for loop
for i in range(len(data)):
    print("Draw bar No.{}, value={}".format(i + 1, data[i]))
    bar_length = data[i] / bar_max_value * bar_max_length  # calculate the bar's length
    x = origin_x
    y = origin_y + (bar_interval + bar_breadth) * i # calculate the y for the current bar
    if data[i] > average:
        color = bar_color_above
    else:
        color = bar_color
    # call draw_horizontal_bar function to draw the bar and text
    draw_horizontal_bar(data[i], x, y, bar_breadth, bar_length, color[0], color[1], color[2])
    wait_mouse_leftclick()

# draw the average bar
average = sum(data) / len(data)
print("Draw the average bar, value={:.3f}".format(average))
bar_length = average / bar_max_value * bar_max_length
y = origin_y + (bar_interval + bar_breadth) * len(data) # caclulate y, instead of writing a fixed number 8, use len(data)
draw_horizontal_bar(average, x, y, bar_breadth, bar_length, bar_color_average[0], bar_color_average[1], bar_color_average[2])
wait_mouse_leftclick()

print("done")

all_done() # always the last instruction of the program
